"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
var react_1 = __importStar(require("react"));
var uuid_1 = require("uuid");
var Transport_1 = require("../Transport");
var RootComponent_1 = require("./RootComponent");
var stringifyWithoutCircular = function (json) {
    // to avoid crash while using circular props
    var getCircularReplacer = function () {
        var seen = new WeakSet();
        return function (_key, value) {
            if (typeof value === "object" && value !== null) {
                if (seen.has(value)) {
                    return;
                }
                seen.add(value);
            }
            return value;
        };
    };
    return JSON.stringify(json, getCircularReplacer());
};
exports.Client = function (props) {
    var _a = react_1.useState({ runningViews: [], }), state = _a[0], setState = _a[1];
    // const transport = useMemo(() => props.transport || createTransport(), [])
    var transport = react_1.useMemo(function () { return Transport_1.createTransport(props.channel); }, []);
    // const context = useContext(ThreadContext) //main
    var UIComponents = react_1.useMemo(function () { return __spreadArrays(props.Components, [RootComponent_1.RootReactMultiThreadedComponent]); }, []);
    react_1.useEffect(function () {
        transport.on("update_views_tree", function (_a) {
            var views = _a.views;
            return setState({ runningViews: views });
        });
        transport.on("update_view", function (_a) {
            var view = _a.view;
            return setState((function () {
                var runningViewIndex = state.runningViews.findIndex(function (currentView) { return currentView.uid === view.uid; });
                if (runningViewIndex !== -1)
                    state.runningViews[runningViewIndex] = view;
                else
                    state.runningViews.push(view);
                return { runningViews: __spreadArrays(state.runningViews) };
            })());
        });
        transport.on("delete_view", function (_a) {
            var viewUid = _a.viewUid;
            var runningViewIndex = state.runningViews.findIndex(function (view) { return view.uid === viewUid; });
            if (runningViewIndex !== -1) {
                state.runningViews.splice(runningViewIndex, 1);
                setState({ runningViews: __spreadArrays(state.runningViews) });
            }
        });
        transport.on("on_worker_start", function () { return transport.emit("request_views_tree"); });
    }, []);
    var renderView = react_1.useCallback(function (view) {
        var componentToRender = UIComponents.find(function (component) { return component.name === view.name; });
        if (componentToRender) {
            var props_1 = { key: view.uid };
            view.props.forEach(function (prop) {
                if (prop.type === "data")
                    props_1[prop.name] = prop.data;
                else if (prop.type === "event")
                    [
                        (props_1[prop.name] = function () {
                            var args = [];
                            for (var _i = 0; _i < arguments.length; _i++) {
                                args[_i] = arguments[_i];
                            }
                            return new Promise(function (resolve) {
                                var requestUid = uuid_1.v4();
                                transport.on("respond_to_event", function (_a) {
                                    var data = _a.data, uid = _a.uid, eventUid = _a.eventUid;
                                    if (uid === requestUid && eventUid === prop.uid)
                                        resolve(data);
                                });
                                transport.emit("request_event", {
                                    eventArguments: JSON.parse(stringifyWithoutCircular(args)),
                                    eventUid: prop.uid,
                                    uid: requestUid,
                                });
                            });
                        }),
                    ];
            });
            var children = state.runningViews
                .filter(function (runningView) { return runningView.parentUid === view.uid; })
                .sort(function (a, b) { return a.childIndex - b.childIndex; })
                .map(function (runningView) { return renderView(runningView); });
            return react_1.default.createElement(componentToRender, __assign(__assign({}, props_1), { children: children.length > 0 ? children : undefined }));
        }
        throw new Error("missing componnent " + view.name + " please pass/register it to the UIComponents array");
    }, [transport, state]);
    var root = state.runningViews.find(function (view) { return view.isRoot; });
    return (!root) ? react_1.default.createElement(react_1.default.Fragment, null) : renderView(root);
};
//# sourceMappingURL=Client.js.map